@@ -15,67 +15,78 @@
 """Flower client app."""
 
 
-import timeit
 from abc import ABC, abstractmethod
-from typing import Dict, List, Optional, Tuple, Union, cast
+from typing import Dict, List, Tuple
 
 import numpy as np
 
 from flwr.common import (
+    Code,
+    Config,
     EvaluateIns,
     EvaluateRes,
     FitIns,
     FitRes,
     Metrics,
     ParametersRes,
+    PropertiesIns,
+    PropertiesRes,
     Scalar,
+    Status,
     parameters_to_weights,
     weights_to_parameters,
 )
 
 from .client import Client
 
-DEPRECATION_WARNING_FIT = """
-DEPRECATION WARNING: deprecated return format
+EXCEPTION_MESSAGE_WRONG_RETURN_TYPE_FIT = """
+NumPyClient.fit did not return a tuple with 3 elements.
+The returned values should have the following type signature:
 
-    parameters, num_examples
+    Tuple[List[np.ndarray], int, Dict[str, Scalar]]
 
-move to
+Example
+-------
 
-    parameters, num_examples, {"custom_key": custom_val}
+    model.get_weights(), 10, {"accuracy": 0.95}
 
-instead. Note that the deprecated return format will be removed in a future
-release.
 """
-DEPRECATION_WARNING_EVALUATE_0 = """
-DEPRECATION WARNING: deprecated return format
 
-    num_examples, loss, accuracy
+EXCEPTION_MESSAGE_WRONG_RETURN_TYPE_EVALUATE = """
+NumPyClient.evaluate did not return a tuple with 3 elements.
+The returned values should have the following type signature:
 
-move to
+    Tuple[float, int, Dict[str, Scalar]]
 
-    loss, num_examples, {"accuracy": accuracy}
+Example
+-------
 
-instead. Note that the deprecated return format will be removed in a future
-release.
-"""
-DEPRECATION_WARNING_EVALUATE_1 = """
-DEPRECATION WARNING: deprecated return format
+    0.5, 10, {"accuracy": 0.95}
 
-    num_examples, loss, accuracy, {"custom_key": custom_val}
-
-move to
-
-    loss, num_examples, {"accuracy": accuracy}
-
-instead. Note that the deprecated return format will be removed in a future
-release.
 """
 
 
 class NumPyClient(ABC):
     """Abstract base class for Flower clients using NumPy."""
 
+    def get_properties(self, config: Config) -> Dict[str, Scalar]:
+        """Returns a client's set of properties.
+
+        Parameters
+        ----------
+        config : Config
+            Configuration parameters requested by the server.
+            This can be used to tell the client which parameters
+            are needed along with some Scalar attributes.
+
+        Returns
+        -------
+        properties : Dict[str, Scalar]
+            A dictionary mapping arbitrary string keys to values of type
+            bool, bytes, float, int, or str. It can be used to communicate
+            arbitrary property values back to the server.
+        """
+
     @abstractmethod
     def get_parameters(self) -> List[np.ndarray]:
         """Return the current local model parameters.
@@ -89,10 +100,7 @@ class NumPyClient(ABC):
     @abstractmethod
     def fit(
         self, parameters: List[np.ndarray], config: Dict[str, Scalar]
-    ) -> Union[
-        Tuple[List[np.ndarray], int, Dict[str, Scalar]],
-        Tuple[List[np.ndarray], int],  # Deprecated
-    ]:
+    ) -> Tuple[List[np.ndarray], int, Dict[str, Scalar]]:
         """Train the provided parameters using the locally held dataset.
 
         Parameters
@@ -120,11 +128,7 @@ class NumPyClient(ABC):
     @abstractmethod
     def evaluate(
         self, parameters: List[np.ndarray], config: Dict[str, Scalar]
-    ) -> Union[
-        Tuple[float, int, Dict[str, Scalar]],
-        Tuple[int, float, float],  # Deprecated
-        Tuple[int, float, float, Dict[str, Scalar]],  # Deprecated
-    ]:
+    ) -> Tuple[float, int, Dict[str, Scalar]]:
         """Evaluate the provided weights using the locally held dataset.
 
         Parameters
@@ -151,18 +155,30 @@ class NumPyClient(ABC):
         Warning
         -------
         The previous return type format (int, float, float) and the
-        extended format (int, float, float, Dict[str, Scalar]) are still
-        supported for compatibility reasons. They will however be removed
-        in a future release, please migrate to (float, int, Dict[str, Scalar]).
+        extended format (int, float, float, Dict[str, Scalar]) have been
+        deprecated and removed since Flower 0.19.
         """
 
 
+def has_get_properties(client: NumPyClient) -> bool:
+    """Check if NumPyClient implements get_properties."""
+    return type(client).get_properties != NumPyClient.get_properties
+
+
 class NumPyClientWrapper(Client):
     """Wrapper which translates between Client and NumPyClient."""
 
     def __init__(self, numpy_client: NumPyClient) -> None:
         self.numpy_client = numpy_client
 
+    def get_properties(self, ins: PropertiesIns) -> PropertiesRes:
+        """Return the current client properties."""
+        properties = self.numpy_client.get_properties(ins.config)
+        return PropertiesRes(
+            status=Status(code=Code.OK, message="Success"),
+            properties=properties,
+        )
+
     def get_parameters(self) -> ParametersRes:
         """Return the current local model parameters."""
         parameters = self.numpy_client.get_parameters()
@@ -175,25 +191,23 @@ class NumPyClientWrapper(Client):
         parameters: List[np.ndarray] = parameters_to_weights(ins.parameters)
 
         # Train
-        fit_begin = timeit.default_timer()
-        results = self.numpy_client.fit(parameters, ins.config)
-        if len(results) == 2:
-            print(DEPRECATION_WARNING_FIT)
-            results = cast(Tuple[List[np.ndarray], int], results)
-            parameters_prime, num_examples = results
-            metrics: Optional[Metrics] = None
-        elif len(results) == 3:
-            results = cast(Tuple[List[np.ndarray], int, Metrics], results)
-            parameters_prime, num_examples, metrics = results
+        results: Tuple[List[np.ndarray], int, Metrics] = self.numpy_client.fit(
+            parameters, ins.config
+        )
+        if not (
+            len(results) == 3
+            and isinstance(results[0], list)
+            and isinstance(results[1], int)
+            and isinstance(results[2], dict)
+        ):
+            raise Exception(EXCEPTION_MESSAGE_WRONG_RETURN_TYPE_FIT)
 
         # Return FitRes
-        fit_duration = timeit.default_timer() - fit_begin
+        parameters_prime, num_examples, metrics = results
         parameters_prime_proto = weights_to_parameters(parameters_prime)
         return FitRes(
             parameters=parameters_prime_proto,
             num_examples=num_examples,
-            num_examples_ceil=num_examples,  # Deprecated
-            fit_duration=fit_duration,  # Deprecated
             metrics=metrics,
         )
 
@@ -201,54 +215,21 @@ class NumPyClientWrapper(Client):
         """Evaluate the provided parameters using the locally held dataset."""
         parameters: List[np.ndarray] = parameters_to_weights(ins.parameters)
 
-        results = self.numpy_client.evaluate(parameters, ins.config)
-        if len(results) == 3:
-            if (
-                isinstance(results[0], float)
-                and isinstance(results[1], int)
-                and isinstance(results[2], dict)
-            ):
-                # Forward-compatible case: loss, num_examples, metrics
-                results = cast(Tuple[float, int, Metrics], results)
-                loss, num_examples, metrics = results
-                evaluate_res = EvaluateRes(
-                    loss=loss,
-                    num_examples=num_examples,
-                    metrics=metrics,
-                )
-            elif (
-                isinstance(results[0], int)
-                and isinstance(results[1], float)
-                and isinstance(results[2], float)
-            ):
-                # Legacy case: num_examples, loss, accuracy
-                # This will be removed in a future release
-                print(DEPRECATION_WARNING_EVALUATE_0)
-                results = cast(Tuple[int, float, float], results)
-                num_examples, loss, accuracy = results
-                evaluate_res = EvaluateRes(
-                    loss=loss,
-                    num_examples=num_examples,
-                    accuracy=accuracy,  # Deprecated
-                )
-            else:
-                raise Exception(
-                    "Return value expected to be of type (float, int, dict)."
-                )
-        elif len(results) == 4:
-            # Legacy case: num_examples, loss, accuracy, metrics
-            # This will be removed in a future release
-            print(DEPRECATION_WARNING_EVALUATE_1)
-            results = cast(Tuple[int, float, float, Metrics], results)
-            assert isinstance(results[0], int)
-            assert isinstance(results[1], float)
-            assert isinstance(results[2], float)
-            assert isinstance(results[3], dict)
-            num_examples, loss, accuracy, metrics = results
-            evaluate_res = EvaluateRes(
-                loss=loss,
-                num_examples=num_examples,
-                accuracy=accuracy,  # Deprecated
-                metrics=metrics,
-            )
-        return evaluate_res
+        results: Tuple[float, int, Metrics] = self.numpy_client.evaluate(
+            parameters, ins.config
+        )
+        if not (
+            len(results) == 3
+            and isinstance(results[0], float)
+            and isinstance(results[1], int)
+            and isinstance(results[2], dict)
+        ):
+            raise Exception(EXCEPTION_MESSAGE_WRONG_RETURN_TYPE_EVALUATE)
+
+        # Return EvaluateRes
+        loss, num_examples, metrics = results
+        return EvaluateRes(
+            loss=loss,
+            num_examples=num_examples,
+            metrics=metrics,
+        )
