@@ -19,22 +19,31 @@ from .parameter import bytes_to_ndarray as bytes_to_ndarray
 from .parameter import ndarray_to_bytes as ndarray_to_bytes
 from .parameter import parameters_to_weights as parameters_to_weights
 from .parameter import weights_to_parameters as weights_to_parameters
+from .typing import Code as Code
+from .typing import Config as Config
 from .typing import Disconnect as Disconnect
 from .typing import EvaluateIns as EvaluateIns
 from .typing import EvaluateRes as EvaluateRes
 from .typing import FitIns as FitIns
 from .typing import FitRes as FitRes
 from .typing import Metrics as Metrics
+from .typing import MetricsAggregationFn as MetricsAggregationFn
 from .typing import Parameters as Parameters
 from .typing import ParametersRes as ParametersRes
+from .typing import Properties as Properties
+from .typing import PropertiesIns as PropertiesIns
+from .typing import PropertiesRes as PropertiesRes
 from .typing import Reconnect as Reconnect
 from .typing import Scalar as Scalar
+from .typing import Status as Status
 from .typing import Weights as Weights
 
 GRPC_MAX_MESSAGE_LENGTH: int = 536_870_912  # == 512 * 1024 * 1024
 
 __all__ = [
     "bytes_to_ndarray",
+    "Code",
+    "Config",
     "Disconnect",
     "EvaluateIns",
     "EvaluateRes",
@@ -42,12 +51,17 @@ __all__ = [
     "FitRes",
     "GRPC_MAX_MESSAGE_LENGTH",
     "Metrics",
+    "MetricsAggregationFn",
     "ndarray_to_bytes",
     "Parameters",
     "parameters_to_weights",
     "ParametersRes",
+    "Properties",
+    "PropertiesIns",
+    "PropertiesRes",
     "Reconnect",
     "Scalar",
+    "Status",
     "Weights",
     "weights_to_parameters",
 ]
