@@ -17,8 +17,8 @@
 
 import concurrent.futures
 import timeit
-from logging import DEBUG, INFO, WARNING
-from typing import Dict, List, Optional, Tuple, Union
+from logging import DEBUG, INFO
+from typing import Dict, List, Optional, Tuple
 
 from flwr.common import (
     Disconnect,
@@ -29,8 +29,6 @@ from flwr.common import (
     Parameters,
     Reconnect,
     Scalar,
-    Weights,
-    weights_to_parameters,
 )
 from flwr.common.logger import log
 from flwr.server.client_manager import ClientManager
@@ -38,54 +36,17 @@ from flwr.server.client_proxy import ClientProxy
 from flwr.server.history import History
 from flwr.server.strategy import FedAvg, Strategy
 
-DEPRECATION_WARNING_EVALUATE = """
-DEPRECATION WARNING: Method
-
-    Server.evaluate(self, rnd: int) -> Optional[
-        Tuple[Optional[float], EvaluateResultsAndFailures]
-    ]
-
-is deprecated and will be removed in a future release, use
-
-    Server.evaluate_round(self, rnd: int) -> Optional[
-        Tuple[Optional[float], Dict[str, Scalar], EvaluateResultsAndFailures]
-    ]
-
-instead.
-"""
-
-DEPRECATION_WARNING_EVALUATE_ROUND = """
-DEPRECATION WARNING: The configured Strategy uses a deprecated aggregate_evaluate
-return format:
-
-    Strategy.aggregate_evaluate(...) -> Optional[float]
-
-This format is deprecated and will be removed in a future release. It should use
-
-    Strategy.aggregate_evaluate(...) -> Tuple[Optional[float], Dict[str, Scalar]]
-
-instead.
-"""
-
-DEPRECATION_WARNING_FIT_ROUND = """
-DEPRECATION WARNING: The configured Strategy uses a deprecated aggregate_fit
-return format:
-
-    Strategy.aggregate_fit(...) -> Optional[Weights]
-
-This format is deprecated and will be removed in a future release. It should use
-
-    Strategy.aggregate_fit(...) -> Tuple[Optional[Weights], Dict[str, Scalar]]
-
-instead.
-"""
-
-FitResultsAndFailures = Tuple[List[Tuple[ClientProxy, FitRes]], List[BaseException]]
+FitResultsAndFailures = Tuple[
+    List[Tuple[ClientProxy, FitRes]],
+    List[BaseException],
+]
 EvaluateResultsAndFailures = Tuple[
-    List[Tuple[ClientProxy, EvaluateRes]], List[BaseException]
+    List[Tuple[ClientProxy, EvaluateRes]],
+    List[BaseException],
 ]
 ReconnectResultsAndFailures = Tuple[
-    List[Tuple[ClientProxy, Disconnect]], List[BaseException]
+    List[Tuple[ClientProxy, Disconnect]],
+    List[BaseException],
 ]
 
 
@@ -100,6 +61,11 @@ class Server:
             tensors=[], tensor_type="numpy.ndarray"
         )
         self.strategy: Strategy = strategy if strategy is not None else FedAvg()
+        self.max_workers: Optional[int] = None
+
+    def set_max_workers(self, max_workers: Optional[int]) -> None:
+        """Set the max_workers used by ThreadPoolExecutor."""
+        self.max_workers = max_workers
 
     def set_strategy(self, strategy: Strategy) -> None:
         """Replace server strategy."""
@@ -110,13 +76,13 @@ class Server:
         return self._client_manager
 
     # pylint: disable=too-many-locals
-    def fit(self, num_rounds: int) -> History:
+    def fit(self, num_rounds: int, timeout: Optional[float]) -> History:
         """Run federated averaging for a number of rounds."""
         history = History()
 
         # Initialize parameters
-        log(INFO, "Getting initial parameters")
-        self.parameters = self._get_initial_parameters()
+        log(INFO, "Initializing global parameters")
+        self.parameters = self._get_initial_parameters(timeout=timeout)
         log(INFO, "Evaluating initial parameters")
         res = self.strategy.evaluate(parameters=self.parameters)
         if res is not None:
@@ -135,7 +101,7 @@ class Server:
 
         for current_round in range(1, num_rounds + 1):
             # Train model and replace previous global model
-            res_fit = self.fit_round(rnd=current_round)
+            res_fit = self.fit_round(rnd=current_round, timeout=timeout)
             if res_fit:
                 parameters_prime, _, _ = res_fit  # fit_metrics_aggregated
                 if parameters_prime:
@@ -157,7 +123,7 @@ class Server:
                 history.add_metrics_centralized(rnd=current_round, metrics=metrics_cen)
 
             # Evaluate model on a sample of available clients
-            res_fed = self.evaluate_round(rnd=current_round)
+            res_fed = self.evaluate_round(rnd=current_round, timeout=timeout)
             if res_fed:
                 loss_fed, evaluate_metrics_fed, _ = res_fed
                 if loss_fed:
@@ -172,20 +138,10 @@ class Server:
         log(INFO, "FL finished in %s", elapsed)
         return history
 
-    def evaluate(
-        self, rnd: int
-    ) -> Optional[Tuple[Optional[float], EvaluateResultsAndFailures]]:
-        """Validate current global model on a number of clients."""
-        log(WARNING, DEPRECATION_WARNING_EVALUATE)
-        res = self.evaluate_round(rnd)
-        if res is None:
-            return None
-        # Deconstruct
-        loss, _, results_and_failures = res
-        return loss, results_and_failures
-
     def evaluate_round(
-        self, rnd: int
+        self,
+        rnd: int,
+        timeout: Optional[float],
     ) -> Optional[
         Tuple[Optional[float], Dict[str, Scalar], EvaluateResultsAndFailures]
     ]:
@@ -206,7 +162,11 @@ class Server:
         )
 
         # Collect `evaluate` results from all clients participating in this round
-        results, failures = evaluate_clients(client_instructions)
+        results, failures = evaluate_clients(
+            client_instructions,
+            max_workers=self.max_workers,
+            timeout=timeout,
+        )
         log(
             DEBUG,
             "evaluate_round received %s results and %s failures",
@@ -215,27 +175,18 @@ class Server:
         )
 
         # Aggregate the evaluation results
-        aggregated_result: Union[
-            Tuple[Optional[float], Dict[str, Scalar]],
-            Optional[float],  # Deprecated
+        aggregated_result: Tuple[
+            Optional[float],
+            Dict[str, Scalar],
         ] = self.strategy.aggregate_evaluate(rnd, results, failures)
 
-        metrics_aggregated: Dict[str, Scalar] = {}
-        if aggregated_result is None:
-            # Backward-compatibility, this will be removed in a future update
-            log(WARNING, DEPRECATION_WARNING_EVALUATE_ROUND)
-            loss_aggregated = None
-        elif isinstance(aggregated_result, float):
-            # Backward-compatibility, this will be removed in a future update
-            log(WARNING, DEPRECATION_WARNING_EVALUATE_ROUND)
-            loss_aggregated = aggregated_result
-        else:
-            loss_aggregated, metrics_aggregated = aggregated_result
-
+        loss_aggregated, metrics_aggregated = aggregated_result
         return loss_aggregated, metrics_aggregated, (results, failures)
 
     def fit_round(
-        self, rnd: int
+        self,
+        rnd: int,
+        timeout: Optional[float],
     ) -> Optional[
         Tuple[Optional[Parameters], Dict[str, Scalar], FitResultsAndFailures]
     ]:
@@ -245,6 +196,7 @@ class Server:
         client_instructions = self.strategy.configure_fit(
             rnd=rnd, parameters=self.parameters, client_manager=self._client_manager
         )
+
         if not client_instructions:
             log(INFO, "fit_round: no clients selected, cancel")
             return None
@@ -256,7 +208,11 @@ class Server:
         )
 
         # Collect `fit` results from all clients participating in this round
-        results, failures = fit_clients(client_instructions)
+        results, failures = fit_clients(
+            client_instructions=client_instructions,
+            max_workers=self.max_workers,
+            timeout=timeout,
+        )
         log(
             DEBUG,
             "fit_round received %s results and %s failures",
@@ -265,31 +221,27 @@ class Server:
         )
 
         # Aggregate training results
-        aggregated_result: Union[
-            Tuple[Optional[Parameters], Dict[str, Scalar]],
-            Optional[Weights],  # Deprecated
+        aggregated_result: Tuple[
+            Optional[Parameters],
+            Dict[str, Scalar],
         ] = self.strategy.aggregate_fit(rnd, results, failures)
 
-        metrics_aggregated: Dict[str, Scalar] = {}
-        if aggregated_result is None:
-            # Backward-compatibility, this will be removed in a future update
-            log(WARNING, DEPRECATION_WARNING_FIT_ROUND)
-            parameters_aggregated = None
-        elif isinstance(aggregated_result, list):
-            # Backward-compatibility, this will be removed in a future update
-            log(WARNING, DEPRECATION_WARNING_FIT_ROUND)
-            parameters_aggregated = weights_to_parameters(aggregated_result)
-        else:
-            parameters_aggregated, metrics_aggregated = aggregated_result
-
+        parameters_aggregated, metrics_aggregated = aggregated_result
         return parameters_aggregated, metrics_aggregated, (results, failures)
 
-    def disconnect_all_clients(self) -> None:
+    def disconnect_all_clients(self, timeout: Optional[float]) -> None:
         """Send shutdown signal to all clients."""
         all_clients = self._client_manager.all()
-        _ = shutdown(clients=[all_clients[k] for k in all_clients.keys()])
+        clients = [all_clients[k] for k in all_clients.keys()]
+        instruction = Reconnect(seconds=None)
+        client_instructions = [(client_proxy, instruction) for client_proxy in clients]
+        _ = reconnect_clients(
+            client_instructions=client_instructions,
+            max_workers=self.max_workers,
+            timeout=timeout,
+        )
 
-    def _get_initial_parameters(self) -> Parameters:
+    def _get_initial_parameters(self, timeout: Optional[float]) -> Parameters:
         """Get initial parameters from one of the available clients."""
 
         # Server-side parameter initialization
@@ -297,26 +249,37 @@ class Server:
             client_manager=self._client_manager
         )
         if parameters is not None:
-            log(INFO, "Received initial parameters from strategy")
+            log(INFO, "Using initial parameters provided by strategy")
             return parameters
 
         # Get initial parameters from one of the clients
+        log(INFO, "Requesting initial parameters from one random client")
         random_client = self._client_manager.sample(1)[0]
-        parameters_res = random_client.get_parameters()
+        parameters_res = random_client.get_parameters(timeout=timeout)
         log(INFO, "Received initial parameters from one random client")
         return parameters_res.parameters
 
 
-def shutdown(clients: List[ClientProxy]) -> ReconnectResultsAndFailures:
+def reconnect_clients(
+    client_instructions: List[Tuple[ClientProxy, Reconnect]],
+    max_workers: Optional[int],
+    timeout: Optional[float],
+) -> ReconnectResultsAndFailures:
     """Instruct clients to disconnect and never reconnect."""
-    reconnect = Reconnect(seconds=None)
-    with concurrent.futures.ThreadPoolExecutor() as executor:
-        futures = [executor.submit(reconnect_client, c, reconnect) for c in clients]
-        concurrent.futures.wait(futures)
+    with concurrent.futures.ThreadPoolExecutor(max_workers=max_workers) as executor:
+        submitted_fs = {
+            executor.submit(reconnect_client, client_proxy, ins, timeout)
+            for client_proxy, ins in client_instructions
+        }
+        finished_fs, _ = concurrent.futures.wait(
+            fs=submitted_fs,
+            timeout=None,  # Handled in the respective communication stack
+        )
+
     # Gather results
     results: List[Tuple[ClientProxy, Disconnect]] = []
     failures: List[BaseException] = []
-    for future in futures:
+    for future in finished_fs:
         failure = future.exception()
         if failure is not None:
             failures.append(failure)
@@ -327,27 +290,38 @@ def shutdown(clients: List[ClientProxy]) -> ReconnectResultsAndFailures:
 
 
 def reconnect_client(
-    client: ClientProxy, reconnect: Reconnect
+    client: ClientProxy,
+    reconnect: Reconnect,
+    timeout: Optional[float],
 ) -> Tuple[ClientProxy, Disconnect]:
-    """Instruct a single client to disconnect and (optionally) reconnect
-    later."""
-    disconnect = client.reconnect(reconnect)
+    """Instruct client to disconnect and (optionally) reconnect later."""
+    disconnect = client.reconnect(
+        reconnect,
+        timeout=timeout,
+    )
     return client, disconnect
 
 
 def fit_clients(
-    client_instructions: List[Tuple[ClientProxy, FitIns]]
+    client_instructions: List[Tuple[ClientProxy, FitIns]],
+    max_workers: Optional[int],
+    timeout: Optional[float],
 ) -> FitResultsAndFailures:
     """Refine parameters concurrently on all selected clients."""
-    with concurrent.futures.ThreadPoolExecutor() as executor:
-        futures = [
-            executor.submit(fit_client, c, ins) for c, ins in client_instructions
-        ]
-        concurrent.futures.wait(futures)
+    with concurrent.futures.ThreadPoolExecutor(max_workers=max_workers) as executor:
+        submitted_fs = {
+            executor.submit(fit_client, client_proxy, ins, timeout)
+            for client_proxy, ins in client_instructions
+        }
+        finished_fs, _ = concurrent.futures.wait(
+            fs=submitted_fs,
+            timeout=None,  # Handled in the respective communication stack
+        )
+
     # Gather results
     results: List[Tuple[ClientProxy, FitRes]] = []
     failures: List[BaseException] = []
-    for future in futures:
+    for future in finished_fs:
         failure = future.exception()
         if failure is not None:
             failures.append(failure)
@@ -358,25 +332,34 @@ def fit_clients(
     return results, failures
 
 
-def fit_client(client: ClientProxy, ins: FitIns) -> Tuple[ClientProxy, FitRes]:
+def fit_client(
+    client: ClientProxy, ins: FitIns, timeout: Optional[float]
+) -> Tuple[ClientProxy, FitRes]:
     """Refine parameters on a single client."""
-    fit_res = client.fit(ins)
+    fit_res = client.fit(ins, timeout=timeout)
     return client, fit_res
 
 
 def evaluate_clients(
-    client_instructions: List[Tuple[ClientProxy, EvaluateIns]]
+    client_instructions: List[Tuple[ClientProxy, EvaluateIns]],
+    max_workers: Optional[int],
+    timeout: Optional[float],
 ) -> EvaluateResultsAndFailures:
     """Evaluate parameters concurrently on all selected clients."""
-    with concurrent.futures.ThreadPoolExecutor() as executor:
-        futures = [
-            executor.submit(evaluate_client, c, ins) for c, ins in client_instructions
-        ]
-        concurrent.futures.wait(futures)
+    with concurrent.futures.ThreadPoolExecutor(max_workers=max_workers) as executor:
+        submitted_fs = {
+            executor.submit(evaluate_client, client_proxy, ins, timeout)
+            for client_proxy, ins in client_instructions
+        }
+        finished_fs, _ = concurrent.futures.wait(
+            fs=submitted_fs,
+            timeout=None,  # Handled in the respective communication stack
+        )
+
     # Gather results
     results: List[Tuple[ClientProxy, EvaluateRes]] = []
     failures: List[BaseException] = []
-    for future in futures:
+    for future in finished_fs:
         failure = future.exception()
         if failure is not None:
             failures.append(failure)
@@ -388,8 +371,10 @@ def evaluate_clients(
 
 
 def evaluate_client(
-    client: ClientProxy, ins: EvaluateIns
+    client: ClientProxy,
+    ins: EvaluateIns,
+    timeout: Optional[float],
 ) -> Tuple[ClientProxy, EvaluateRes]:
     """Evaluate parameters on a single client."""
-    evaluate_res = client.evaluate(ins)
+    evaluate_res = client.evaluate(ins, timeout=timeout)
     return client, evaluate_res
