@@ -25,6 +25,7 @@ import numpy as np
 
 from flwr.common import (
     FitRes,
+    MetricsAggregationFn,
     Parameters,
     Scalar,
     Weights,
@@ -36,6 +37,7 @@ from flwr.server.client_proxy import ClientProxy
 from .fedopt import FedOpt
 
 
+# pylint: disable=too-many-locals
 class FedAdagrad(FedOpt):
     """Adaptive Federated Optimization using Adagrad (FedAdagrad) [Reddi et
     al., 2020] strategy.
@@ -57,6 +59,8 @@ class FedAdagrad(FedOpt):
         ] = None,
         on_fit_config_fn: Optional[Callable[[int], Dict[str, Scalar]]] = None,
         on_evaluate_config_fn: Optional[Callable[[int], Dict[str, Scalar]]] = None,
+        fit_metrics_aggregation_fn: Optional[MetricsAggregationFn] = None,
+        evaluate_metrics_aggregation_fn: Optional[MetricsAggregationFn] = None,
         accept_failures: bool = True,
         initial_parameters: Parameters,
         eta: float = 1e-1,
@@ -65,7 +69,7 @@ class FedAdagrad(FedOpt):
     ) -> None:
         """Federated learning strategy using Adagrad on server-side.
 
-        Implementation based on https://arxiv.org/abs/2003.00295
+        Implementation based on https://arxiv.org/abs/2003.00295v5
 
         Args:
             fraction_fit (float, optional): Fraction of clients used during
@@ -87,6 +91,10 @@ class FedAdagrad(FedOpt):
             accept_failures (bool, optional): Whether or not accept rounds
                 containing failures. Defaults to True.
             initial_parameters (Parameters): Initial set of parameters from the server.
+            fit_metrics_aggregation_fn: Optional[MetricsAggregationFn]
+                Metrics aggregation function, optional.
+            evaluate_metrics_aggregation_fn: Optional[MetricsAggregationFn]
+                Metrics aggregation function, optional.
             eta (float, optional): Server-side learning rate. Defaults to 1e-1.
             eta_l (float, optional): Client-side learning rate. Defaults to 1e-1.
             tau (float, optional): Controls the algorithm's degree of adaptability.
@@ -103,11 +111,14 @@ class FedAdagrad(FedOpt):
             on_evaluate_config_fn=on_evaluate_config_fn,
             accept_failures=accept_failures,
             initial_parameters=initial_parameters,
+            fit_metrics_aggregation_fn=fit_metrics_aggregation_fn,
+            evaluate_metrics_aggregation_fn=evaluate_metrics_aggregation_fn,
             eta=eta,
             eta_l=eta_l,
+            beta_1=0.0,
+            beta_2=0.0,
             tau=tau,
         )
-        self.v_t: Optional[Weights] = None
 
     def __repr__(self) -> str:
         rep = f"FedAdagrad(accept_failures={self.accept_failures})"
@@ -126,27 +137,30 @@ class FedAdagrad(FedOpt):
         if fedavg_parameters_aggregated is None:
             return None, {}
 
-        fedavg_aggregate = parameters_to_weights(fedavg_parameters_aggregated)
-        aggregated_updates = [
-            subset_weights - self.current_weights[idx]
-            for idx, subset_weights in enumerate(fedavg_aggregate)
-        ]
+        fedavg_weights_aggregate = parameters_to_weights(fedavg_parameters_aggregated)
 
         # Adagrad
-        delta_t = aggregated_updates
-        if not self.v_t:
-            self.v_t = [np.zeros_like(subset_weights) for subset_weights in delta_t]
-
-        self.v_t = [
-            self.v_t[idx] + np.multiply(subset_weights, subset_weights)
-            for idx, subset_weights in enumerate(delta_t)
+        delta_t = [
+            x - y for x, y in zip(fedavg_weights_aggregate, self.current_weights)
         ]
 
+        # m_t
+        if not self.m_t:
+            self.m_t = [np.zeros_like(x) for x in delta_t]
+        self.m_t = [
+            self.beta_1 * x + (1 - self.beta_1) * y for x, y in zip(self.m_t, delta_t)
+        ]
+
+        # v_t
+        if not self.v_t:
+            self.v_t = [np.zeros_like(x) for x in delta_t]
+        self.v_t = [x + np.multiply(y, y) for x, y in zip(self.v_t, delta_t)]
+
         new_weights = [
-            self.current_weights[idx]
-            + self.eta * delta_t[idx] / (np.sqrt(self.v_t[idx]) + self.tau)
-            for idx in range(len(delta_t))
+            x + self.eta * y / (np.sqrt(z) + self.tau)
+            for x, y, z in zip(self.current_weights, self.m_t, self.v_t)
         ]
+
         self.current_weights = new_weights
 
         return weights_to_parameters(self.current_weights), metrics_aggregated
