@@ -16,8 +16,8 @@
 
 
 import statistics
-from logging import INFO
-from typing import Callable, Dict, List, Optional, Tuple
+from logging import INFO, WARNING
+from typing import Callable, Dict, List, Optional, Tuple, cast
 
 import numpy as np
 
@@ -25,6 +25,7 @@ from flwr.common import (
     EvaluateRes,
     FitIns,
     FitRes,
+    MetricsAggregationFn,
     Parameters,
     Scalar,
     Weights,
@@ -67,6 +68,8 @@ class FedFSv0(FedAvg):
         t_fast: int = 10,
         t_slow: int = 10,
         initial_parameters: Optional[Parameters] = None,
+        fit_metrics_aggregation_fn: Optional[MetricsAggregationFn] = None,
+        evaluate_metrics_aggregation_fn: Optional[MetricsAggregationFn] = None,
     ) -> None:
         super().__init__(
             fraction_fit=fraction_fit,
@@ -78,6 +81,8 @@ class FedFSv0(FedAvg):
             on_fit_config_fn=on_fit_config_fn,
             on_evaluate_config_fn=on_evaluate_config_fn,
             initial_parameters=initial_parameters,
+            fit_metrics_aggregation_fn=fit_metrics_aggregation_fn,
+            evaluate_metrics_aggregation_fn=evaluate_metrics_aggregation_fn,
         )
         self.min_completion_rate_fit = min_completion_rate_fit
         self.min_completion_rate_evaluate = min_completion_rate_evaluate
@@ -86,6 +91,8 @@ class FedFSv0(FedAvg):
         self.t_fast = t_fast
         self.t_slow = t_slow
         self.contributions: Dict[str, List[Tuple[int, int, int]]] = {}
+        self.fit_metrics_aggregation_fn = fit_metrics_aggregation_fn
+        self.evaluate_metrics_aggregation_fn = evaluate_metrics_aggregation_fn
 
     def __repr__(self) -> str:
         rep = f"FedFSv0(r_fast={self.r_fast}, r_slow={self.r_slow}, "
@@ -146,7 +153,7 @@ class FedFSv0(FedAvg):
         for idx, (cid, _) in enumerate(all_clients.items()):
             cid_idx[idx] = cid
             penalty = 0.0
-            if cid in self.contributions.keys():
+            if cid in self.contributions:
                 contribs: List[Tuple[int, int, int]] = self.contributions[cid]
                 penalty = statistics.mean([c / m for _, c, m in contribs])
             # `p` should be:
@@ -191,17 +198,30 @@ class FedFSv0(FedAvg):
         # Track contributions to the global model
         for client, fit_res in results:
             cid = client.cid
-            assert fit_res.num_examples_ceil is not None
+
+            assert "num_examples_ceil" in fit_res.metrics
+            num_examples_ceil: int = cast(int, fit_res.metrics["num_examples_ceil"])
+
             contribution: Tuple[int, int, int] = (
                 rnd,
                 fit_res.num_examples,
-                fit_res.num_examples_ceil,
+                num_examples_ceil,
             )
-            if cid not in self.contributions.keys():
+            if cid not in self.contributions:
                 self.contributions[cid] = []
             self.contributions[cid].append(contribution)
 
-        return weights_to_parameters(weights_prime), {}
+        parameters_aggregated = weights_to_parameters(weights_prime)
+
+        # Aggregate custom metrics if aggregation fn was provided
+        metrics_aggregated = {}
+        if self.fit_metrics_aggregation_fn:
+            fit_metrics = [(res.num_examples, res.metrics) for _, res in results]
+            metrics_aggregated = self.fit_metrics_aggregation_fn(fit_metrics)
+        elif rnd == 1:
+            log(WARNING, "No fit_metrics_aggregation_fn provided")
+
+        return parameters_aggregated, metrics_aggregated
 
     def aggregate_evaluate(
         self,
@@ -219,16 +239,20 @@ class FedFSv0(FedAvg):
             # Not enough results for aggregation
             return None, {}
 
-        return (
-            weighted_loss_avg(
-                [
-                    (
-                        evaluate_res.num_examples,
-                        evaluate_res.loss,
-                        evaluate_res.accuracy,
-                    )
-                    for client, evaluate_res in results
-                ]
-            ),
-            {},
+        # Aggregate loss
+        loss_aggregated = weighted_loss_avg(
+            [
+                (evaluate_res.num_examples, evaluate_res.loss)
+                for _, evaluate_res in results
+            ]
         )
+
+        # Aggregate custom metrics if aggregation fn was provided
+        metrics_aggregated = {}
+        if self.evaluate_metrics_aggregation_fn:
+            eval_metrics = [(res.num_examples, res.metrics) for _, res in results]
+            metrics_aggregated = self.evaluate_metrics_aggregation_fn(eval_metrics)
+        elif rnd == 1:
+            log(WARNING, "No evaluate_metrics_aggregation_fn provided")
+
+        return loss_aggregated, metrics_aggregated
